==> emodpy_covid-0.0.31-py3-none-any.whl: emodpy_covid/mid/schema_to_class.py <==
import json
import os
from collections import OrderedDict
import pdb

class ReadOnlyDict(OrderedDict):
    def __missing__(self, key):
        raise KeyError(f"'{key}' not found in this object.")

    def __getattr__(self, item):
        return self[item]

    def __setattr__(self, key, value):
        #if key not in self and "Config" not in key and "List" not in key: # these are lame; find way in schema to initialize complex types {}, [], or null
        if key not in self: # these are lame; find way in schema to initialize complex types {}, [], or null
            self.__missing__(key) # this should not be necessary

        self[key] = value
        return

def get_default_campaign_event():
    event = {}
    event = json.loads(
        """
        {
            "Event_Coordinator_Config": {},
            "Nodeset_Config": { "class": "NodeSetAll" },
            "Start_Day": -1,
            "class": "CampaignEvent"
            }
        """,
        object_hook=ReadOnlyDict
    )
    return event


def get_class_with_defaults( classname, schema_path="schema.json" ):
    """
    Very handy little class -- that needs a lot more error handling and robustification --
    that gets you the default config for an event coordinator or intervention from the schema.
    """
    if not os.path.exists( schema_path ):
        print( f"No file found at {schema_path}. A valid schema path needs to exist at the path specified." )
        return None

    ret_json = {}
    schema = json.loads( open( schema_path ).read() )
    if "coordinator" in classname.lower():
        for ec_name in schema["idmTypes"]["idmAbstractType:EventCoordinator"].keys():
            if classname.replace( "EventCoordinator", "" ) in ec_name:
                #print( schema["idmTypes"]["idmAbstractType:EventCoordinator"][ec_name] )
                schema_blob = schema["idmTypes"]["idmAbstractType:EventCoordinator"][ec_name]
                ret_json["class"] = schema_blob["class"]
                for ec_key in schema_blob.keys():
                    try:
                        if "default" in schema_blob[ec_key]:
                            ret_json[ec_key] = schema_blob[ec_key]["default"]
                            # have to hack a couple of things for now, TBD: fix in schema
                            if ec_key == "Property_Restrictions_Within_Node" or ec_key == "Node_Property_Restrictions":
                                ret_json[ec_key] = []
                    except Exception as ex:
                        print( str( ex ) )
                break # once we find it, stop
    elif "waning" in classname.lower():
        #for wan_type in schema["idmTypes"]["idmType:WaningEffect"].keys():
        if classname in schema["idmTypes"]["idmType:WaningEffect"].keys():
            schema_blob = schema["idmTypes"]["idmType:WaningEffect"][classname]
            ret_json["class"] = schema_blob["class"]
            for wan_key in schema_blob.keys():
                try:
                    if "default" in schema_blob[wan_key]:
                        ret_json[wan_key] = schema_blob[wan_key]["default"]
                except Exception as ex:
                    print( str( ex ) )
    else:
        #print( schema["idmTypes"]["idmAbstractType:Intervention"].keys() )
        for iv_type in schema["idmTypes"]["idmAbstractType:Intervention"].keys():
            if classname in schema["idmTypes"]["idmAbstractType:Intervention"][iv_type].keys():
                #print( schema["idmTypes"]["idmAbstractType:Intervention"][iv_type][classname] )
                schema_blob = schema["idmTypes"]["idmAbstractType:Intervention"][iv_type][classname]
                ret_json["class"] = schema_blob["class"]
                for iv_key in schema_blob.keys():
                    try:
                        if "default" in schema_blob[iv_key]:
                            ret_json[iv_key] = schema_blob[iv_key]["default"]
                            if iv_key == "Property_Restrictions_Within_Node" or iv_key == "Node_Property_Restrictions":
                                ret_json[iv_key] = []
                    except Exception as ex:
                        print( str( ex ) )
                break
    return ReadOnlyDict( ret_json )

==> emodpy_covid-0.0.50-py3-none-any.whl: emodpy_covid/mid/schema_to_class.py <==
import json
import os
from collections import OrderedDict
import pdb

class ReadOnlyDict(OrderedDict):
    def __missing__(self, key):
        raise KeyError(f"'{key}' not found in this object.")

    def __getattr__(self, item):
        return self[item]

    def __setattr__(self, key, value):
        #if key not in self and "Config" not in key and "List" not in key: # these are lame; find way in schema to initialize complex types {}, [], or null
        if key not in self: # these are lame; find way in schema to initialize complex types {}, [], or null
            self.__missing__(key) # this should not be necessary

        if "schema" in self:
            if self["schema"][key]["type"] in [ "integer", "float" ]:
                if value < self["schema"][key]["min"]:
                    print( "{} is below Minimum {}.".format( value, self["schema"][key]["min"] ) )
                    return
                elif value > self["schema"][key]["max"]:
                    print( "{} is above Maximum {}.".format( value, self["schema"][key]["max"] ) )
                    return
            elif self["schema"][key]["type"] == "enum":
                if value not in self["schema"][key]["enum"]:
                    print( "{} is not list of possible values {}.".format( value, self["schema"][key]["enum"] ) )
                    return

        self[key] = value
        return

    def set_schema(self,schema):
        #self.schema = schema
        self["schema"] = schema

    def finalize(self):
        self.pop( "schema" )
        return self

def get_default_campaign_event():
    event = {}
    event = json.loads(
        """
        {
            "Event_Coordinator_Config": {},
            "Nodeset_Config": { "class": "NodeSetAll" },
            "Start_Day": -1,
            "class": "CampaignEvent"
            }
        """,
        object_hook=ReadOnlyDict
    )
    return event


def get_class_with_defaults( classname, schema_path="schema.json" ):
    """
    Very handy little class -- that needs a lot more error handling and robustification --
    that gets you the default config for an event coordinator or intervention from the schema.
    """
    if not os.path.exists( schema_path ):
        print( f"No file found at {schema_path}. A valid schema path needs to exist at the path specified." )
        return None

    ret_json = {}
    schema = json.loads( open( schema_path ).read() )
    schema_blob = None
    if "coordinator" in classname.lower():
        for ec_name in schema["idmTypes"]["idmAbstractType:EventCoordinator"].keys():
            if classname.replace( "EventCoordinator", "" ) in ec_name:
                #print( schema["idmTypes"]["idmAbstractType:EventCoordinator"][ec_name] )
                schema_blob = schema["idmTypes"]["idmAbstractType:EventCoordinator"][ec_name]
                ret_json["class"] = schema_blob["class"]
                for ec_key in schema_blob.keys():
                    try:
                        if "default" in schema_blob[ec_key]:
                            ret_json[ec_key] = schema_blob[ec_key]["default"]
                            # have to hack a couple of things for now, TBD: fix in schema
                            if ec_key == "Property_Restrictions_Within_Node" or ec_key == "Node_Property_Restrictions":
                                ret_json[ec_key] = []
                    except Exception as ex:
                        print( str( ex ) )
                break # once we find it, stop
    elif "waning" in classname.lower():
        #for wan_type in schema["idmTypes"]["idmType:WaningEffect"].keys():
        if classname in schema["idmTypes"]["idmType:WaningEffect"].keys():
            schema_blob = schema["idmTypes"]["idmType:WaningEffect"][classname]
            ret_json["class"] = schema_blob["class"]
            for wan_key in schema_blob.keys():
                try:
                    if "default" in schema_blob[wan_key]:
                        ret_json[wan_key] = schema_blob[wan_key]["default"]
                except Exception as ex:
                    print( str( ex ) )
    else:
        #print( schema["idmTypes"]["idmAbstractType:Intervention"].keys() )
        for iv_type in schema["idmTypes"]["idmAbstractType:Intervention"].keys():
            if classname in schema["idmTypes"]["idmAbstractType:Intervention"][iv_type].keys():
                #print( schema["idmTypes"]["idmAbstractType:Intervention"][iv_type][classname] )
                schema_blob = schema["idmTypes"]["idmAbstractType:Intervention"][iv_type][classname]
                ret_json["class"] = schema_blob["class"]
                for iv_key in schema_blob.keys():
                    try:
                        if "default" in schema_blob[iv_key]:
                            ret_json[iv_key] = schema_blob[iv_key]["default"]
                            if iv_key == "Property_Restrictions_Within_Node" or iv_key == "Node_Property_Restrictions":
                                ret_json[iv_key] = []
                    except Exception as ex:
                        print( str( ex ) )
                break
    ret_this = ReadOnlyDict( ret_json )
    #ret_this.schema = schema_blob 
    ret_this.set_schema( schema_blob )
    return ret_this
